@@ -10,7 +10,6 @@ use wasm_bus_types::SerializationFormat;
 
 use crate::abi::CallError;
 use crate::abi::CallHandle;
-use crate::task::spawn;
 
 type CallbackHandler = Arc<
     dyn Fn(CallHandle, Vec<u8>) -> Pin<Box<dyn Future<Output = Result<Vec<u8>, CallError>> + Send>>
@@ -58,36 +57,31 @@ impl RespondToService {
         callbacks.remove(handle)
     }
 
-    pub fn process(&self, callback_handle: CallHandle, handle: CallHandle, request: Vec<u8>) {
+    pub async fn process(&self, callback_handle: CallHandle, handle: CallHandle, request: Vec<u8>) {
         let callback = {
             let callbacks = self.callbacks.lock().unwrap();
             if let Some(callback) = callbacks.get(&callback_handle) {
                 Arc::clone(callback)
             } else {
-                spawn(async move {
-                    let err: u32 = CallError::InvalidHandle.into();
-                    crate::abi::syscall::fault(handle, err as u32);
-                    crate::engine::BusEngine::remove(&handle, "invalid callback handle");
-                });
+                let err: u32 = CallError::InvalidHandle.into();
+                crate::abi::syscall::fault(handle, err as u32);
+                crate::engine::BusEngine::remove(&handle, "invalid callback handle");
                 return;
             }
         };
 
-        let persistent = self.persistent;
-        spawn(async move {
-            let res = callback.as_ref()(handle, request);
-            match res.await {
-                Ok(a) => {
-                    crate::abi::syscall::reply(handle, &a[..]);
-                }
-                Err(err) => {
-                    let err: u32 = err.into();
-                    crate::abi::syscall::fault(handle, err as u32);
-                }
+        let res = callback.as_ref()(handle, request);
+        match res.await {
+            Ok(a) => {
+                crate::abi::syscall::reply(handle, &a[..]);
             }
-            if persistent == false {
-                crate::engine::BusEngine::remove(&handle, "request was processed (by respond_to)");
+            Err(err) => {
+                let err: u32 = err.into();
+                crate::abi::syscall::fault(handle, err as u32);
             }
-        });
+        }
+        if self.persistent == false {
+            crate::engine::BusEngine::remove(&handle, "request was processed (by respond_to)");
+        }
     }
 }
